package Catalyst::Authentication::Credential::Devel;
use Moose;
use namespace::autoclean;

with 'MooseX::Emulate::Class::Accessor::Fast';

__PACKAGE__->mk_accessors(
    qw/source user realm username_field/);

sub new {
    my ($class, $config, $app, $realm) = @_;

    my $self = { };
    bless $self, $class;

    $self->source($config->{source} || 'REMOTE_USER');
    $self->user($config->{user} || '');
    $self->realm($realm);
    $self->username_field($config->{username_field} || 'username');
    return $self;
}

sub authenticate {
    my ($self, $c, $realm, $authinfo) = @_;

    my $remuser;
    # Value from configuration takes precedence over environment variable
    if ($self->user) {
        $remuser = $self->user;
    }
    else {
        $remuser = $ENV{$self->source};
    }

    return unless defined($remuser);
    return if ($remuser eq "");

    # $authinfo hash can contain item username (it is optional) - if it is so
    # this username has to be equal to remote_user
    my $authuser = $authinfo->{username};
    return if (defined($authuser) && ($authuser ne $remuser));

    $authinfo->{ $self->username_field } = $remuser;
    my $user_obj = $realm->find_user( $authinfo, $c );

    return ref($user_obj) ? $user_obj : undef;
}

1;

__END__

=pod

=head1 NAME

Catalyst::Authentication::Credential::Devel - Let developer manually
choose Catalyst application users

=head1 SYNOPSIS

    # in your MyApp.pm
    __PACKAGE__->config(

        'Plugin::Authentication' => {
            default_realm => 'remoterealm',
            realms => {
                remoterealm => {
                    credential => {
                        class  => 'Devel',
                        source => 'SSL_CLIENT_S_DN',
                        user   => 'user@company.org',
                    },
                    store => {
                        class => 'Null',
                        # if you want to have some additional user attributes
                        # like user roles, user full name etc. you can specify
                        # here the store where you keep this data
                    }
                },
            },
        },

    );

=head1 DESCRIPTION

This module allows developer to manually specify, which user is currently
authenticated.

=head1 CONFIGURATION

=head2 class

This config item is B<REQUIRED>.

B<class> is part of the core L<Catalyst::Plugin::Authentication> module, it
contains the class name of the store to be used.

The classname used for Credential. This is part of L<Catalyst::Plugin::Authentication>
and is the method by which Catalyst::Authentication::Credential::Remote is
loaded as the credential validator. For this module to be used, this must be set
to 'Remote'.

=head2 source

This config item is B<OPTIONAL> - default is REMOTE_USER.

B<source> contains a name of a environment variable, from which to read the user name.

=head2 user

This config item is B<OPTIONAL> - default is ''.

B<user> contains a name of a user, which should be considered as logged in. This
configuration takes precedence over B<source>.

=cut
