package Mentat::Stats::Dashboard;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Stats::Dashboard - Short module description

=head1 SYNOPSIS

 use Mentat::Stats::Dashboard;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Constants;
use Mentat::Storage::Mongo::Aggregator;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant GRAPH_PIE              => "pie";
use constant GRAPH_BAR              => "bar";
use constant DFLT_BASEPATH          => '/data/dashboard/';
use constant DFLT_FILEPATH          => '/var/mentat/reports/dashboard/';
use constant DFLT_FILEEXT           => '.png';
use constant DFLT_COMPRESSION       => 0; # 9 max compression, 0 no compression
use constant DFLT_GRAPH_WIDTH_SRC   => 400;
use constant DFLT_GRAPH_HEIGHT_SRC  => 400;
use constant DFLT_GRAPH_WIDTH_DST   => 400;
use constant DFLT_GRAPH_HEIGHT_DST  => 400;
use constant DFLT_GRAPH_3D          => 0;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $BASEPATH            = DFLT_BASEPATH();
my $FILEPATH            = DFLT_FILEPATH();
my $FILEEXT             = DFLT_FILEEXT();
my $COMPRESSION         = DFLT_COMPRESSION();
my $GRAPH_WIDTH_SRC     = DFLT_GRAPH_WIDTH_SRC();
my $GRAPH_HEIGHT_SRC    = DFLT_GRAPH_HEIGHT_SRC();
my $GRAPH_WIDTH_DST     = DFLT_GRAPH_WIDTH_DST();
my $GRAPH_HEIGHT_DST    = DFLT_GRAPH_HEIGHT_DST();
my $GRAPH_3D            = DFLT_GRAPH_3D();

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Stats::Dashboard->new();
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Stats::Dashboard reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Mentat.Stats.Dashboard instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Mentat.Stats.Dashboard instance...
}

#-------------------------------------------------------------------------------

=item aggregator() [PUBLIC]

  Usage     : my $aggregator = $instance->aggregator();
  Purpose   : Get reference to the internal Mentat::Storage::Mongo::Aggregator
  Arguments : None
  Returns   : Mentat::Storage::Mongo::Aggregator $aggregator
  Throws    : Dies, if not invoked on object instance

=cut

sub aggregator()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->{AGGREGATOR};
}

=item get_intervals() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :

=cut

sub get_intervals()
{
    my $self    = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless ref $self;

    my ($sec, $min, $hour, $mday, $mon, $year) = localtime();
    my $today_str = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, 0, 0, 0);
    my $today = Value::Convertor->datestr_to_unixs($today_str);

    my $intervals = [
        {
            to => Value::Convertor->unixs_to_datestr($today, 1),
            from => Value::Convertor->unixs_to_datestr($today - Mentat::Constants::TIME_SEC_DAY(), 1),
            text => "1D,",
        },
        {
            to => Value::Convertor->unixs_to_datestr($today, 1),
            from => Value::Convertor->unixs_to_datestr($today - 3 * Mentat::Constants::TIME_SEC_DAY(), 1),
            text => "3D,",
        },
        {
            to => Value::Convertor->unixs_to_datestr($today, 1),
            from => Value::Convertor->unixs_to_datestr($today - Mentat::Constants::TIME_SEC_WEEK(), 1),
            text => "1W,",
        },
        {
            to => Value::Convertor->unixs_to_datestr($today, 1),
            from => Value::Convertor->unixs_to_datestr($today - 2 * Mentat::Constants::TIME_SEC_WEEK(), 1),
            text => "2W,",
        },
        {
            to => Value::Convertor->unixs_to_datestr($today, 1),
            from => Value::Convertor->unixs_to_datestr($today - Mentat::Constants::TIME_SEC_MONTH(), 1),
            text => "1M",
        },
#        {
#            to => Value::Convertor->unixs_to_datestr($today, 1),
#            from => Value::Convertor->unixs_to_datestr($today - 3 * Mentat::Constants::TIME_SEC_MONTH(), 1),
#            #text => "3 months",
#            text => "3M,",
#        },
#        {
#            to => Value::Convertor->unixs_to_datestr($today, 1),
#            from => Value::Convertor->unixs_to_datestr($today - 6 * Mentat::Constants::TIME_SEC_MONTH(), 1),
#            text => "6 months",
#        },
#        {
#            to => Value::Convertor->unixs_to_datestr($today, 1),
#            from => Value::Convertor->unixs_to_datestr($today - Mentat::Constants::TIME_SEC_YEAR(), 1),
#            text => "1 year",
#        },
    ];

    return $intervals;
}

=item get_table($$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :

=cut

sub get_table($$$)
{
    my $self    = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless ref $self;

    my ($query, $from, $to) = @_;

    my $tables;
    my $dashboad_data = $self->{AGGREGATOR}->get_dashboard_data($query, $from, $to);

    foreach my $dd (@{$dashboad_data})
    {

        my $table = $dd->{table};
        my @src_array;

        foreach my $image (@{$dd->{image}})
        {
            $image->{filename} .= $FILEEXT;

            my $result = $self->generate_graph_image($image);
            if (defined $result)
            {
                push (@src_array, $BASEPATH.$image->{filename});
            }
        }

        if (@src_array > 0) {
            push (@$tables,{
                title_column => $table->{title_column},
                title_page => $table->{title_page},
                id => $table->{id},
                table => $table->{data},
                src => \@src_array,
            });
        }
    }

    return $tables;
}

=item generate_graph_image($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :

=cut

sub generate_graph_image($)
{
    my $self    = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless ref $self;

    my ($image) = @_;
    my $graph = $image->{data};
    my $my_graph;

    if ($image->{graph_type} eq GRAPH_PIE())
    {
        $my_graph = new GD::Graph::pie( $GRAPH_WIDTH_SRC, $GRAPH_HEIGHT_SRC );
        $my_graph->set(
            'title'     => $image->{title_image},
            'label'     => $image->{x_label},
            'dclrs'     => $image->{palette},
            '3d'        => $GRAPH_3D,
            'bgclr'     => 'gray',
            'accentclr' => 'white',
        );
    }
    else
    {
        my $legend = shift @$graph;
        my $g = shift @$graph;
        $graph = $g;

        $my_graph = GD::Graph::bars->new($GRAPH_WIDTH_SRC, $GRAPH_HEIGHT_SRC);
        $my_graph->set_legend(@$legend);
        $my_graph->set(
            'title'         => $image->{title_image},
            'x_label'       => $image->{x_label},
            'y_label'       => $image->{y_label},
            'dclrs'         => $image->{palette},
            'x_labels_vertical' => 't',
            'long_ticks'    => 1,
            'tick_length'   => 0,
            'x_ticks'       => 0,
            'x_label_position' => .5,
            'y_label_position' => .5,
            'cumulate'      => 1,

            'bgclr'         => 'white',
            'transparent'   => 0,
            'interlaced'    => 1,
        );
    }

    if (defined $graph && defined $graph->[0]->[0])
    {

        my $gd = $my_graph->plot($graph);
        $self->_save_image($image->{filename}, $gd->png($COMPRESSION));

        return $image->{filename};
    }

    return undef;
}


#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Stats::Dashboard instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Stats::Dashboard $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Mentat.Stats.Dashboard instance...
    my %config = @_;

    $self->{AGGREGATOR} = Mentat::Storage::Mongo::Aggregator->new(%config);

    return $self;
}


# _save_image($$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    :
#  Comments  :
#  See Also  :

sub _save_image($$)
{
    my $self    = shift;
    croak ((caller(0))[3] . ": instance method invoked on class") unless ref $self;

    my ($filename, $data) = @_;

    open(IMG, ">$FILEPATH$filename") or die $!;
    binmode IMG;
    print IMG $data;
    close(IMG);
}

=pod

=back

=cut

1;

