package Hawat::Controller::Brief;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Brief',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Brief - Catalyst controller for event reports

=head1 DESCRIPTION

Catalyst controller for handling actions related to event reports.

=head1 METHODS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('view')));
}

=head2 view

View the details of one particular report

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    my $operation = $c->request->params->{submit};
    if ($id) {
        $c->stash(oper => 'fetching');
    }
    elsif ($operation) {
        $c->stash(oper => 'calculating');
    }

    $c->stash(template => 'brief/view.html.tt');
    
    # Configure and forward to '/dbitem/fetch' action
    #my $conf = $self->_dbicnf(
        #regexp_id => '^[-_a-zA-Z0-9]+$',
    #);
    #$c->forward(HAWAT_DBI_FETCH, [$conf, $id, $c->cfg('briefs_dir')]);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the devel switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Secure for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 delete

Delete one particular report

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    #my $backlink = $c->session->{backlink} || $c->uri_for('/brief');

    # Configure and forward to '/dbitem/fetch' action
    #my $conf = $self->_dbicnf(
        #regexp_id  => '^[-_a-zA-Z0-9]+$',
        #backlink   => $backlink,
        #backlink_d => $c->uri_for('/brief/list'),
    #);
    #$c->forward(HAWAT_DBI_DELETE, [$conf, $id, $confirm, $c->cfg('briefs_dir')]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Secure for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 calculate

Calculate the brief for certain time period

=cut

#sub calculate :Local :Args(1) { # Switch to let the framework handle the error reporting
sub calculate :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    #$c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    #my $conf = $self->_dbicnf(
        #regexp_id => '^[-_a-zA-Z0-9]+$',
    #);
    #$c->forward(HAWAT_DBI_FETCH, [$conf, $id, $c->cfg('briefs_dir')]);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the devel switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    my $date = $c->request->params->{brief_date};
    my $type = $c->request->params->{brief_type};

    my $brief = $c->model('Briefs')->calculate($date, $type, $c->cfg('briefs_dir'));
    $c->stash(%$brief);

    $c->stash(template => 'brief/view.html.tt');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Secure for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details

#sub COMPONENT {
#   my ($class, $app, $args) = @_;
#   $args = $class->merge_config_hashes($class->config, $args);
#   #use Data::Dumper;
#   #$app->log->debug("X $class: " . Dumper($class->config));
#   #$app->log->debug("Y args  : " . Dumper($args));
#   #$app->log->debug("Z $app  : " . Dumper($app->config));
#   #$app->log->debug("MERGE XY: " . Dumper($args));
#   return $class->new($app, $args);
#}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Brief',
            model      => HAWAT_MODEL_REPORTS(),
            items      => 'Briefs',
            item       => 'Brief',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
